/***************************************************************************
*   Copyright (C) 2001 by Rick L. Vinyard, Jr.                            *
*   rvinyard@cs.nmsu.edu                                                  *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU Lesser General Public License as        *
*   published by the Free Software Foundation version 2.1.                *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU Lesser General Public      *
*   License along with this library; if not, write to the                 *
*   Free Software Foundation, Inc.,                                       *
*   51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA              *
***************************************************************************/
#include "recordstorage.h"

namespace bit
  {

  RecordStorage::RecordStorage(FieldBase& record): m_record(&record)
  {}


  RecordStorage::~RecordStorage()
  {}

  FieldBase::iterator RecordStorage::begin()
  {
    return FieldBase::iterator( m_record, *(field_set.begin()) );
  }

  FieldBase::iterator RecordStorage::end()
  {
    return FieldBase::iterator( m_record, FieldBase::pointer() );
  }

  RecordStorage& RecordStorage::operator=(const RecordStorage& other) {
    field_set.clear();
    field_name_map.clear();
    FieldBase::pointer field;
    Fields::const_iterator i;

    for (i = other.field_set.begin(); i != other.field_set.end(); i++) {
      field = (*i)->clone();
      // TODO put in a proper throw
      if ( field->m_parent != NULL )
        throw;
      add_field(field);
    }

    return *this;
  }

  FieldBase::pointer RecordStorage::field(std::string s) {
    size_t split = s.find_first_of(".");

    if (split == std::string::npos )
    {
      FieldNameMap::iterator iter = field_name_map.find(s);
      if (iter == field_name_map.end() )
        return FieldBase::pointer();
      else
        return *(iter->second);
    }
    else
    {
      std::string field, remainder;
      field = s.substr(0, split);
      remainder = s.substr(split+1, s.size());
      FieldNameMap::iterator iter = field_name_map.find(field);
      if ( iter == field_name_map.end() )
        return FieldBase::pointer();
      else
      {
        FieldBase::pointer p = *(iter->second);
        return p->field(remainder);
      }
    }
  }

  FieldBase::pointer RecordStorage::field(size_t i) {
    if ( i >= field_set.size() )
      return FieldBase::pointer();

    Fields::iterator iter = field_set.begin();
    std::advance(iter, i);
    return *iter;
  }

  void RecordStorage::add_field(FieldBase::pointer field)
  {
    // TODO put in a proper throw
    if ( field->m_parent != NULL )
      throw;

    field->m_parent = m_record;

    Fields::iterator iter = field_set.insert(field).first;
    field_name_map[ field->name() ] = iter;
  }

  void RecordStorage::remove_field(const std::string& name)
  {
    FieldNameMap::iterator i = field_name_map.find(name);

    if ( i == field_name_map.end() )
      return;

    FieldBase::pointer field = *(i->second);
    // clear field parent
    field->m_parent = NULL;

    field_name_map.erase( field->name() );
    field_set.erase( field );
  }

  void RecordStorage::remove_field(FieldBase::pointer field)
  {
    if ( field_set.find(field) == field_set.end() )
      return;

    // clear field parent
    field->m_parent = NULL;

    field_name_map.erase( field->name() );
    field_set.erase( field );
  }

  FieldBase::pointer RecordStorage::previous_field(FieldBase::pointer current_field) throw (exception::invalid_container_op)
  {
    if ( ! current_field )
      return FieldBase::pointer();

    Fields::iterator iter = field_set.find(current_field);
    if ( iter == field_set.end() || iter == field_set.begin() )
      return FieldBase::pointer();

    return *(--iter);
  }

  FieldBase::pointer RecordStorage::next_field(FieldBase::pointer current_field) throw (exception::invalid_container_op)
  {

    if ( ! current_field )
      return FieldBase::pointer();

    Fields::iterator iter = field_set.find(current_field);
    if ( iter != field_set.end() )
      iter++;
    if ( iter == field_set.end() )
      return FieldBase::pointer();
    else
      return *iter;
  }

  size_t RecordStorage::fields()
  {
    return field_set.size();
  }



}
